#!/bin/sh

set -o errexit
smallsizes="1 2 64 128 1024 8192 65536"
largesizes="$smallsizes 8388608 67108864 536870912"

NPROC=${NPROC:-8}

if [ $NPROC -gt 4 ] ; then
    sizes="$smallsizes"
else
    sizes="$largesizes"
fi

make_file() {
    if [ $1 -lt 512 ] ; then
        count=$1
        bs=1
    else
        bs=512
        count=`expr $1 / 512`
    fi
    dd if=/dev/zero of=$2 count=$count bs=$bs > /dev/null 2>&1
}

run_file_tests() {
    set +x
    indir=$1/in
    outdir=$1/out.$2
    mkdir -p $outdir
    for size in $sizes ; do
        infile=$indir/file.$size.in
        outfile=$outdir/file.$size.out
#        curl -s -T $infile -o $outfile http://localhost:10000 || { echo error $! uploading file $infile ; exit 1 ; }
        
#        cmp -s $infile $outfile || { echo error files $infile and $outfile differ ; exit 1 ; }
        python put.py $infile || exit 1
    done
}

testdir=`mktemp -d`
indir=$testdir/in
if [ ! -d $indir ] ; then mkdir -p $indir ; fi
for size in $sizes ; do
    infile=$indir/file.$size.in
    make_file $size $infile
done

ii=$NPROC
while [ $ii -gt 0 ] ; do
    echo proc $ii
    run_file_tests $testdir $ii &
    ii=`expr $ii - 1`
done || : # why does while fail????
echo waiting
wait || echo no procs to wait for
rm -rf $testdir
echo done
