
#include <string.h>
#include <signal.h>

#include "nspr.h"
#include "plgetopt.h"
#include "plstr.h"
#include "ns_thrpool.h"
#include "ns_tls.h"
#include "ns_event_fw.h"

#include "test-protocol.h"

static void
test_shutdown(ns_job_t *job)
{
    fprintf(stderr, "Received shutdown signal\n");

    /* Stop our listener job first. */
    ns_job_done((ns_job_t *)job->data);

    /* Shutdown the thread pool. */
    ns_thrpool_shutdown(job->tp);

    /* NGK - this might leak, but we can't really finish the job if the event loop is gone! */
    /* Finish the shutdown signal job. */
    ns_job_done(job);
}

int
main (int argc, char **argv)
{
    ns_thrpool_t *tp;
    struct ns_sec_ctx_t *sc = NULL;
    char *progname = argv[0];
    PRInt32 maxthreads = 3;
#ifdef USE_SSL
    char *secdir, *certname, *prefix;
#endif

    (void)progname; /* TODO */
    ns_job_t *listen_job = NULL;
    argv++;

    if (getenv("MAX_THREADS")) {
        maxthreads = atoi(getenv("MAX_THREADS"));
    }
    tp = ns_thrpool_new(maxthreads, maxthreads, 0, 1024);

#ifdef USE_SSL /* NGK - disabled ssl for testing */
    secdir = *argv++;
    certname = *argv++;
    prefix = NULL;
    sc = ns_tls_init(secdir, prefix, certname, PR_FALSE);
#endif

    /* arguments are listen ports and/or unix domain listen file names */
    for (; argv && *argv; ++argv) {
        PRFileDesc *listenfd = NULL;
        PRNetAddr netaddr;
        int portnum = atoi(*argv);
        PRSocketOptionData prsod = {PR_SockOpt_Nonblocking, {PR_TRUE}};

        if (portnum == 0) { /* unix socket */
            netaddr.local.family = PR_AF_LOCAL;
            PL_strncpyz(netaddr.local.path, *argv, sizeof(netaddr.local.path));
        } else {
            netaddr.inet.family = PR_AF_INET;
            netaddr.inet.port = PR_htons(portnum);
            netaddr.inet.ip = PR_htonl(PR_INADDR_ANY);
        }
        listenfd = PR_OpenTCPSocket(PR_NetAddrFamily(&netaddr));
        PR_SetSocketOption(listenfd, &prsod);
        prsod.option = PR_SockOpt_Reuseaddr;
        PR_SetSocketOption(listenfd, &prsod);
        /*        add_sec_layer(&listenfd, sc);*/
        PR_Bind(listenfd, &netaddr);
        PR_Listen(listenfd, 32);

        /* start the listener job */
        listen_job = test_protocol_init(tp, listenfd, sc);

        /* set up our signal handler for handling shutdown */
        ns_add_signal_job(tp, SIGTERM, NS_JOB_THREAD, test_shutdown, listen_job, NULL);
    }

    if (ns_thrpool_wait(tp)) {
        /* error */
    }

    ns_thrpool_destroy(tp);

/* NGK - disabled SSL for testing
    ns_tls_done(sc);
*/
    fprintf(stderr, "Shutdown.  Exiting.\n");

    return (0);
}
